@@ -5,7 +5,6 @@
 #ifndef GEMMI_DENCALC_HPP_
 #define GEMMI_DENCALC_HPP_
 
-#include <cassert>
 #include "addends.hpp"  // for Addends
 #include "formfact.hpp" // for ExpSum
 #include "grid.hpp"     // for Grid
@@ -21,25 +20,25 @@ Real determine_cutoff_radius(Real x1, const ExpSum<N, Real>& precal, Real cutoff
   // Generally, density is supposed to decrease with radius.
   // But if we have addends (in particular -Z for Mott-Bothe),
   // it can first rise, then decrease. We want to be after the maximum.
-  while (dy > 0) { // unlikely
+  while (std::copysign(dy, y1 * dy) > 0) { // unlikely
     x1 += 1.0f;
     std::tie(y1, dy) = precal.calculate_with_derivative(x1);
   }
   Real x2 = x1;
   Real y2 = y1;
-  if (y1 < cutoff_level) {
-    while (y1 < cutoff_level) {
+  if (std::fabs(y1) < cutoff_level) {
+    while (std::fabs(y1) < cutoff_level) {
       x2 = x1;
       y2 = y1;
       x1 -= 0.5f;
       std::tie(y1, dy) = precal.calculate_with_derivative(x1);
       // with addends it's possible to land on the left side of the maximum
-      if (dy > 0) { // unlikely
-        while (dy > 0 && x1 + 0.1f < x2) {
+      if (std::copysign(dy, y1 * dy) > 0) { // unlikely
+        while (std::copysign(dy, y1 * dy) > 0 && x1 + 0.1f < x2) {
           x1 += 0.1f;
           std::tie(y1, dy) = precal.calculate_with_derivative(x1);
         }
-        if (y1 < cutoff_level)
+        if (std::fabs(y1) < cutoff_level)
           return x1;
         break;
       }
@@ -50,7 +49,7 @@ Real determine_cutoff_radius(Real x1, const ExpSum<N, Real>& precal, Real cutoff
       }
     }
   } else {
-    while (y2 > cutoff_level) {
+    while (std::fabs(y2) > cutoff_level) {
       x1 = x2;
       y1 = y2;
       x2 += 0.5f;
@@ -105,7 +104,8 @@ struct DensityCalculator {
   // pre: check if Table::has(atom.element)
   void add_atom_density_to_grid(const Atom& atom) {
     Element el = atom.element;
-    do_add_atom_density_to_grid(atom, Table::get(el, atom.charge), addends.get(el));
+    const auto& coef = Table::get(el, atom.charge, atom.serial);
+    do_add_atom_density_to_grid(atom, coef, addends.get(el));
   }
 
   // Parameter c is a constant factor and has the same meaning as either addend
