@@ -20,6 +20,20 @@ namespace gemmi {
 #pragma GCC diagnostic ignored "-Wfloat-conversion"
 #endif
 
+template<class Real>
+struct ZeroCoef {
+  using Coef = GaussianCoef<0, 1, Real>;
+  static Real data[121];
+
+  static Real& get_(El el) { return data[static_cast<int>(el)]; }
+  static bool has(El el) { return static_cast<size_t>(el) < sizeof(data) / sizeof(Real); }
+  static Coef get(El el, signed char /*charge*/=0, int /*serial*/=0) { return Coef{{get_(el)}}; }
+};
+
+// real part of the bound coherent neutron scattering length (fm)
+template<class Real>
+Real ZeroCoef<Real>::data[121] = { /*X*/ 0.0 };
+
 template<class Real>
 struct Neutron92 {
   using Coef = GaussianCoef<0, 1, Real>;
@@ -27,7 +41,7 @@ struct Neutron92 {
 
   static Real& get_(El el) { return data[static_cast<int>(el)]; }
   static bool has(El el) { return get_(el) != 0; }
-  static Coef get(El el, signed char /*charge*/=0) { return Coef{{get_(el)}}; }
+  static Coef get(El el, signed char /*charge*/=0, int /*serial*/=0) { return Coef{{get_(el)}}; }
 };
 
 // real part of the bound coherent neutron scattering length (fm)
