@@ -5,15 +5,13 @@
 #ifndef GEMMI_RECGRID_HPP_
 #define GEMMI_RECGRID_HPP_
 
+#include <memory>  // for unique_ptr
 #include "asudata.hpp"
 #include "grid.hpp"
 
 namespace gemmi {
 
-inline signed char friedel_mate_value(signed char v) { return v; }
-inline float friedel_mate_value(float v) { return v; }
-inline double friedel_mate_value(double v) { return v; }
-
+template<typename T> T friedel_mate_value(T v) { return v; }
 template<typename T>
 std::complex<T> friedel_mate_value(const std::complex<T>& v) {
   return std::conj(v);
@@ -106,6 +104,10 @@ struct ReciprocalGrid : GridBase<T> {
     AsuData<R> asu_data;
     if (this->axis_order == AxisOrder::ZYX)
       fail("get_asu_values(): ZYX order is not supported yet");
+    // Why "- 1" below? To skip the value at Nyquist frequency (±n/2).
+    // For even lengths of DFT (real -> reciprocal space) the resulting
+    // h=+nu/2 and h=-nu/2 are both represented by one (strictly real) value.
+    // The grid should be big enough so that these values are not needed.
     int max_h = (this->nu - 1) / 2;
     int max_k = (this->nv - 1) / 2;
     int max_l = half_l ? this->nw - 1 : (this->nw - 1) / 2;
