@@ -29,6 +29,7 @@ enum class ResidueKind : unsigned char {
 };
 
 struct ResidueInfo {
+  char name[8];
   ResidueKind kind;
   // linking type: 0=n/a, 1=peptide-linking, 2=nucl.-linking, 3=(2|1)
   std::uint8_t linking_type;
@@ -58,7 +59,10 @@ struct ResidueInfo {
   bool is_na_linking() const { return (linking_type & 2); }
 };
 
-GEMMI_DLL ResidueInfo find_tabulated_residue(const std::string& name);
+GEMMI_DLL ResidueInfo& get_residue_info(size_t idx);
+GEMMI_DLL size_t find_tabulated_residue_idx(const std::string& name);
+constexpr size_t unknown_tabulated_residue_idx() { return 367; };
+GEMMI_DLL ResidueInfo& find_tabulated_residue(const std::string& name);
 
 /// kind can be AA, RNA or DNA
 inline const char* expand_one_letter(char c, ResidueKind kind) {
@@ -95,9 +99,7 @@ inline const char* expand_protein_one_letter(char c) {
   return expand_one_letter(c, ResidueKind::AA);
 }
 // deprecated
-inline std::vector<std::string> expand_protein_one_letter_string(const std::string& s) {
-  return expand_one_letter_sequence(s, ResidueKind::AA);
-}
+GEMMI_DLL std::vector<std::string> expand_protein_one_letter_string(const std::string& s);
 
 
 } // namespace gemmi
