@@ -22,7 +22,7 @@ namespace cif = gemmi::cif;
 namespace {
 
 enum OptionIndex {
-  FormatIn=AfterMonLibOptions, Sort, Update, RemoveH, KeepH, Water, Unique, NoChange
+  FormatIn=AfterMonLibOptions, Sort, Update, DFraction, RemoveH, KeepH, Water, Unique, NoChange
 };
 
 const option::Descriptor Usage[] = {
@@ -45,6 +45,8 @@ const option::Descriptor Usage[] = {
   { Update, 0, "", "update", Arg::None,
     "  --update  \tIf deprecated atom names (from _chem_comp_atom.alt_atom_id)"
     " are used in the model, change them." },
+  { DFraction, 0, "", "d-fract", Arg::Float,
+    "  --d-fract=FRACT \tAdd atoms as H/D mix, with the given deuterium fraction." },
   { NoOp, 0, "", "", Arg::None,
     "Hydrogen options, mutually exclusive. Default: add hydrogens, but not to water." },
   { Water, 0, "", "water", Arg::None,
@@ -92,22 +94,25 @@ int GEMMI_MAIN(int argc, char **argv) {
   else if (p.options[RemoveH])
     h_change = gemmi::HydrogenChange::Remove;
 
-  MonArguments mon_args;
-  if (h_change != gemmi::HydrogenChange::Remove)
-    mon_args = get_monomer_args(p.options);
-
-  if (p.options[Verbose])
-    std::printf("Reading coordinates from %s\n", input.c_str());
   gemmi::CoorFormat input_format = coor_format_as_enum(p.options[FormatIn]);
   if (input_format == gemmi::CoorFormat::Unknown)
     input_format = gemmi::coor_format_from_ext_gz(input);
   gemmi::CoorFormat output_format = gemmi::coor_format_from_ext_gz(output);
   bool preserve_doc = (input_format == gemmi::CoorFormat::Mmcif &&
                        output_format == gemmi::CoorFormat::Mmcif);
+
+  MonArguments mon_args;
+  if (h_change != gemmi::HydrogenChange::Remove) {
+    bool needs_monlib = input_format != gemmi::CoorFormat::ChemComp;
+    mon_args = get_monomer_args(p.options, needs_monlib);
+  }
+
+  if (p.options[Verbose])
+    std::printf("Reading coordinates from %s\n", input.c_str());
   try {
     gemmi::Structure st;
     std::unique_ptr<cif::Document> doc;
-    if (preserve_doc)
+    if (preserve_doc || mon_args.has_plus())
       doc.reset(new cif::Document);
     st = gemmi::read_structure_gz(input, input_format, doc.get());
     if (st.models.empty() || st.models[0].chains.empty()) {
@@ -126,23 +131,26 @@ int GEMMI_MAIN(int argc, char **argv) {
       read_monomer_lib_and_user_files(monlib, wanted, mon_args, doc.get());
       if (!wanted.empty())
         gemmi::fail("Please create definitions for missing monomers.");
-      if (p.options[Update]) {
-        std::string msg = monlib.update_old_atom_names(st);
-        std::printf("%s", msg.c_str());
-      }
+      if (p.options[Update])
+        monlib.update_old_atom_names(st, {&gemmi::Logger::to_stdout});
       for (size_t i = 0; i != st.models.size(); ++i) {
         // preparing topology modifies hydrogens in the model
-        prepare_topology(st, monlib, i, h_change, p.options[Sort], &std::cerr);
+        prepare_topology(st, monlib, i, h_change, p.options[Sort], {&gemmi::Logger::to_stderr});
       }
     }
+    if (p.options[DFraction]) {
+      double d_fract = std::atof(p.options[DFraction].arg);
+      gemmi::set_deuterium_fraction_of_hydrogens(st, (float)d_fract);
+    }
     if (p.options[Verbose])
       std::printf("Hydrogen site count: %zu in input, %zu in output.\n",
                   initial_h, count_hydrogens(st));
     if (p.options[Verbose])
       std::printf("Writing coordinates to %s\n", output.c_str());
     gemmi::Ofstream os(output, &std::cout);
-    if (gemmi::coor_format_from_ext_gz(output) == gemmi::CoorFormat::Pdb) {
+    if (output_format == gemmi::CoorFormat::Pdb) {
       shorten_ccd_codes(st);
+      gemmi::store_deuterium_as_fraction(st, false);
       gemmi::write_pdb(st, os.ref());
     } else {
       if (preserve_doc) {
