@@ -4,7 +4,7 @@
 
 #include <cstdio>
 #include <cstdlib>            // for strtod
-#include <iostream>           // for cout, cerr
+#include <iostream>           // for cout
 #include <gemmi/mtz2cif.hpp>
 #include <gemmi/gz.hpp>       // for MaybeGzipped
 #include <gemmi/fstream.hpp>  // for Ofstream
@@ -12,6 +12,7 @@
 #include <gemmi/intensit.hpp> // for Intensities
 #include <gemmi/read_cif.hpp> // for read_cif_gz
 #include <gemmi/xds_ascii.hpp>
+#include "gemmi/refln.hpp"
 #define GEMMI_PROG mtz2cif
 #include "options.h"
 
@@ -114,14 +115,15 @@ int GEMMI_MAIN(int argc, char **argv) {
   OptParser p(EXE_NAME);
   p.simple_parse(argc, argv, Usage);
   if (p.options[PrintSpec]) {
-    std::printf("                                for merged mtz\n");
+    std::printf(" ========   for merged mtz   ========\n");
     const char** lines = gemmi::MtzToCif::default_spec(/*for_merged=*/true);
     for (; *lines != nullptr; ++lines)
       std::printf("%s\n", *lines);
-    std::printf("\n                             for unmerged mtz\n");
+    std::printf("\n ========   for unmerged mtz   ========\n");
     lines = gemmi::MtzToCif::default_spec(/*for_merged=*/false);
     for (; *lines != nullptr; ++lines)
       std::printf("%s\n", *lines);
+    std::printf("\n");
     return 0;
   }
 
@@ -153,7 +155,7 @@ int GEMMI_MAIN(int argc, char **argv) {
     }
     std::swap(cif_input, mtz_paths[0]);
   }
-  if (mtz_paths[nargs-2] && gemmi::giends_with(mtz_paths[nargs-2], ".hkl"))
+  if (mtz_paths[nargs-2] && gemmi::giends_with(mtz_paths[nargs-2], "hkl" /*.hkl or .ahkl*/))
     std::swap(xds_path, mtz_paths[nargs-2]);
   if (gemmi::giends_with(cif_output, ".mtz")) {
     std::fprintf(stderr, "This must be a mistake, requested output cif file"
@@ -165,7 +167,7 @@ int GEMMI_MAIN(int argc, char **argv) {
       mtz[i].reset(new gemmi::Mtz);
       if (verbose) {
         std::fprintf(stderr, "Reading %s ...\n", mtz_paths[i]);
-        mtz[i]->warnings = &std::cerr;
+        mtz[i]->logger.callback = gemmi::Logger::to_stderr;
       }
       try {
         mtz[i]->read_input(gemmi::MaybeGzipped(mtz_paths[i]), true);
@@ -176,8 +178,7 @@ int GEMMI_MAIN(int argc, char **argv) {
     }
   if (xds_path) {
     try {
-      xds_ascii.reset(new gemmi::XdsAscii);
-      xds_ascii->read_input(gemmi::MaybeGzipped(xds_path));
+      xds_ascii.reset(new gemmi::XdsAscii(gemmi::read_xds_ascii(xds_path)));
       xds_ascii->gather_iset_statistics();
     } catch (std::runtime_error& e) {
       std::fprintf(stderr, "ERROR reading %s: %s\n", xds_path, e.what());
@@ -262,40 +263,43 @@ int GEMMI_MAIN(int argc, char **argv) {
   }
 
   bool ok = true;
+  gemmi::Logger logger{&gemmi::Logger::to_stderr};
   if (mtz[0] && mtz_to_cif.spec_lines.empty())
-    remove_appendix_from_column_names(*mtz[0], std::cerr);
+    remove_appendix_from_column_names(*mtz[0], logger);
   if (check_merged_columns && mtz[0])
-    ok = gemmi::validate_merged_mtz_deposition_columns(*mtz[0], std::cerr);
+    ok = gemmi::validate_merged_mtz_deposition_columns(*mtz[0], logger);
   gemmi::Intensities mi;
   if (mtz[0])
     mtz_to_cif.staraniso_version = mi.take_staraniso_b_from_mtz(*mtz[0]);
   if (validate && nargs == 3) {
     try {
       if (mtz[0]) {
-        if (!mtz_to_cif.staraniso_version.empty()) {
-          std::cerr << "Merged MTZ went through STARANISO " << mtz_to_cif.staraniso_version;
-          if (mi.staraniso_b.ok())
-            std::cerr << ". Taking into account anisotropic scaling.\n";
-          else
-            std::cerr << ". B tensor is unknown. Intensities won't be checked.\n";
-        }
-        mi.read_merged_intensities_from_mtz(*mtz[0]);
+        if (!mtz_to_cif.staraniso_version.empty())
+          std::fprintf(stderr, "Merged MTZ went through STARANISO %s. %s.\n",
+                       mtz_to_cif.staraniso_version.c_str(),
+                       mi.staraniso_b.ok() ? "Taking into account anisotropic scaling"
+                                           : "B tensor is unknown. Intensities won't be checked");
+        mi.import_mtz(*mtz[0], gemmi::DataType::MergedAM);
       } else {
         gemmi::ReflnBlock rblock = gemmi::get_refln_block(
-            gemmi::read_cif_from_buffer(cif_buf, cif_input).blocks, {});
+            gemmi::read_cif_from_memory(cif_buf.data(), cif_buf.size(), cif_input).blocks,
+            {});
         if (!rblock.entry_id.empty() && rblock.entry_id != mtz_to_cif.entry_id) {
           fprintf(stderr, "Note: using _entry.id from mmCIF (%s) instead of %s.\n",
                   rblock.entry_id.c_str(), mtz_to_cif.entry_id.c_str());
           mtz_to_cif.entry_id = rblock.entry_id;
         }
         mi.take_staraniso_b_from_mmcif(rblock.block);
-        mi.read_merged_intensities_from_mmcif(rblock);
+        mi.import_refln_block(rblock, gemmi::DataType::MergedAM);
       }
       gemmi::Intensities ui;
-      if (mtz[1])
-        ui.read_unmerged_intensities_from_mtz(*mtz[1]);
-      else if (xds_ascii)
-        ui.read_unmerged_intensities_from_xds(*xds_ascii);
+      if (mtz[1]) {
+        ui.import_mtz(*mtz[1], gemmi::DataType::Unmerged);
+      } else if (xds_ascii) {
+        if (xds_ascii->read_columns < 8)
+          gemmi::fail("XDS file contains merged data");
+        ui.import_xds(*xds_ascii);
+      }
 
       bool relaxed_check =
         p.options[NoIntensityCheck] ||
@@ -303,7 +307,7 @@ int GEMMI_MAIN(int argc, char **argv) {
         // allow intensities to differ.
         (!mtz_to_cif.staraniso_version.empty() && !mi.staraniso_b.ok());
 
-      if (!gemmi::validate_merged_intensities(mi, ui, relaxed_check, std::cerr))
+      if (!gemmi::validate_merged_intensities(mi, ui, relaxed_check, logger))
         ok = false;
     } catch (std::exception& e) {
       fprintf(stderr, "Error. Intensities could not be validated.\n%s.\n", e.what());
