@@ -2,16 +2,17 @@
 
 #define GEMMI_PROG na
 #include "options.h"
-#include <cstdio>   // for fprintf, fopen
+#include <cstdio>   // for fprintf, stdin
 #include <cstdlib>  // for strtol, strtod, exit
 #include <cstring>  // for strcmp, strchr
+#include <gemmi/atox.hpp>      // for skip_blank
 #include <gemmi/fileutil.hpp>  // for file_open_or
+#include <gemmi/gz.hpp>        // for zlib_version
+#include <gemmi/model.hpp>     // for CoorFormat
 #include <gemmi/pdb_id.hpp>    // for expand_if_pdb_code
+#include <gemmi/to_cif.hpp>    // for cif::WriteOptions
+#include <gemmi/util.hpp>      // for trim_str
 #include <gemmi/version.hpp>   // for GEMMI_VERSION
-#include <gemmi/util.hpp>   // for trim_str
-#include <gemmi/model.hpp>  // for gemmi::CoorFormat
-#include <gemmi/to_cif.hpp>  // for gemmi::cif::Style
-#include <gemmi/atox.hpp>   // for skip_blank
 
 using std::fprintf;
 
@@ -47,6 +48,25 @@ std::vector<double> parse_blank_separated_numbers(const char* arg) {
   return results;
 }
 
+template<typename T, typename Func>
+bool parse_number_or_range_(const Func& strto, const char* start, T* value1, T* value2) {
+  char* endptr = nullptr;
+  *value1 = strto(start, &endptr);
+  *value2 = *value1;
+  if (endptr != start && *endptr == ':') {
+    start = endptr + 1;
+    *value2 = strto(start, &endptr);
+  }
+  return endptr != start && *endptr == '\0';
+}
+
+bool parse_number_or_range(const char* start, float* value1, float* value2) {
+  return parse_number_or_range_(&std::strtof, start, value1, value2);
+}
+bool parse_number_or_range(const char* start, double* value1, double* value2) {
+  return parse_number_or_range_(&std::strtod, start, value1, value2);
+}
+
 option::ArgStatus Arg::Required(const option::Option& option, bool msg) {
   if (option.arg != nullptr)
     return option::ARG_OK;
@@ -151,6 +171,16 @@ option::ArgStatus Arg::Float3(const option::Option& option, bool msg) {
   return option::ARG_ILLEGAL;
 }
 
+option::ArgStatus Arg::NumberOrRange(const option::Option& option, bool msg) {
+  double tmp;
+  if (option.arg && parse_number_or_range(option.arg, &tmp, &tmp))
+    return option::ARG_OK;
+  if (msg)
+    fprintf(stderr, "Option -%s requires a single number or a range denoted as number:number\n",
+                    given_name(option));
+  return option::ARG_ILLEGAL;
+}
+
 // we wrap fwrite because passing it directly may cause warning
 // "ignoring attributes on template argument" [-Wignored-attributes]
 static
@@ -173,7 +203,7 @@ void OptParser::simple_parse(int argc, char** argv,
     std::exit(0);
   }
   if (options[Version]) {
-    print_version(program_name);
+    print_version(program_name, options[Verbose]);
     std::exit(0);
   }
   if (options[NoOp]) {
@@ -242,18 +272,14 @@ OptParser::paths_from_args_or_file(int opt, int other) {
   if (file_option) {
     if (nonOptionsCount() > other)
       print_try_help_and_exit("Error: File arguments together with option -f.");
-    std::FILE *f = std::fopen(file_option.arg, "r");
-    if (!f) {
-      std::perror(file_option.arg);
-      std::exit(2);
-    }
+
+    auto f = gemmi::file_open_or(file_option.arg, "r", stdin);
     char buf[512];
-    while (std::fgets(buf, 512, f)) {
+    while (std::fgets(buf, 512, f.get())) {
       std::string s = gemmi::trim_str(buf);
       if (!s.empty())
         paths.emplace_back(s);
     }
-    std::fclose(f);
   } else {
     require_input_files_as_args(other);
     for (int i = other; i < nonOptionsCount(); ++i)
@@ -272,28 +298,50 @@ gemmi::CoorFormat coor_format_as_enum(const option::Option& format_in) {
       format = gemmi::CoorFormat::Pdb;
     else if (eq("json") || eq("mmjson"))
       format = gemmi::CoorFormat::Mmjson;
-    else if (eq("chemcomp"))
+    else if (std::strncmp(format_in.arg, "chemcomp", 8) == 0 &&
+             (format_in.arg[8] == '\0' || format_in.arg[8] == ':'))
       format = gemmi::CoorFormat::ChemComp;
   }
   return format;
 }
 
-gemmi::cif::Style cif_style_as_enum(const option::Option& cif_style) {
+gemmi::cif::WriteOptions cif_write_options(const option::Option& cif_style) {
+  gemmi::cif::WriteOptions options;
+  options.prefer_pairs = true;
   if (cif_style)
     switch (cif_style.arg[0]) {
       // value for 'd' (default) is returned at end of this function
-      case 'p'/*pdbx*/: return gemmi::cif::Style::Pdbx;
-      case 'a'/*aligned*/: return gemmi::cif::Style::Aligned;
+      case 'p'/*pdbx*/:
+        options.misuse_hash = true;
+        break;
+      case 'a'/*aligned*/:
+        options.align_pairs = 33;
+        options.align_loops = 30;
+        break;
     }
-  return gemmi::cif::Style::PreferPairs;
+  return options;
 }
 
-void print_version(const char* program_name) {
+void print_version(const char* program_name, bool verbose) {
   std::printf("%s " GEMMI_VERSION
 #ifdef GEMMI_VERSION_INFO
          " (" GEMMI_XSTRINGIZE(GEMMI_VERSION_INFO) ")"
 #endif
          "\n", program_name);
+  if (verbose) {
+#if defined(_MSC_VER)
+    std::printf("Compiler: MSVC %d (C++ %ld)\n", _MSC_FULL_VER, _MSVC_LANG);
+#else
+#  if defined(__clang__)
+    std::printf("Compiler: Clang %d.%d.%d (C++ %ld)\n",
+                __clang_major__, __clang_minor__, __clang_patchlevel__, __cplusplus);
+#  elif defined(__GNUC__)
+    std::printf("Compiler: GCC %d.%d.%d (C++ %ld)\n",
+                __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__, __cplusplus);
+#  endif
+    std::printf("Built with: %s\n", gemmi::zlib_description);
+#endif
+  }
 }
 
 void read_spec_file(const char* path, std::vector<std::string>& output) {
