@@ -31,9 +31,11 @@
 namespace {
 
 enum OptionIndex {
-  Hkl=4, Dmin, For, NormalizeIt92, Rate, Blur, RCut, Test, ToMtz, Compare,
-  CifFp, Wavelength, Unknown, NoAniso, Margin, ScaleTo, SigmaCutoff, FLabel,
-  PhiLabel, Ksolv, Bsolv, Baniso, RadiiSet, Rprobe, Rshrink, WriteMap
+  Hkl=4, Dmin, For, NormalizeIt92, UseCharge, Rate, Blur, RCut,
+  Test, WriteMap, ToMtz, Compare, FLabel, PhiLabel, Anomalous,
+  CifFp, Wavelength, Unknown, NoAniso, Margin, ScaleTo, SigmaCutoff,
+  MaskSpacing, RadiiSet, Rprobe, Rshrink, MaskFile, Ksolv, Bsolv, Kov, Baniso,
+  FCanomLabel, PHICanomLabel
 };
 
 struct SfCalcArg: public Arg {
@@ -84,11 +86,15 @@ const option::Descriptor Usage[] = {
     "  --for=TYPE  \tTYPE is xray (default), electron, neutron or mott-bethe." },
   { NormalizeIt92, 0, "", "normalize-it92", Arg::None,
     "  --normalize-it92  \tNormalize X-ray form factors (a tiny change)." },
+  { UseCharge, 0, "", "use-charge", Arg::None,
+    "  --use-charge  \tUse X-ray form factors with charges when available." },
   { CifFp, 0, "", "ciffp", Arg::None,
     "  --ciffp  \tRead f' from _atom_type_scat_dispersion_real in CIF." },
   { Wavelength, 0, "w", "wavelength", Arg::Float,
     "  --wavelength=NUM  \tWavelength [A] for calculation of f' "
     "(use --wavelength=0 or -w0 to ignore anomalous scattering)." },
+  { Anomalous, 0, "", "anomalous", Arg::None,
+    "  --anomalous  \tCalculate anomalous map (f\")." },
   { Unknown, 0, "", "unknown", Arg::Required,
     "  --unknown=SYMBOL  \tUse form factor of SYMBOL for unknown atoms." },
   { NoAniso, 0, "", "noaniso", Arg::None,
@@ -119,17 +125,24 @@ const option::Descriptor Usage[] = {
     "  --sigma-cutoff=NUM  \tUse only data with F/SIGF > NUM (default: 0)." },
   // TODO: solvent option: mask, babinet, none
 
-  { NoOp, 0, "", "", Arg::None, "\nOptions for bulk solvent correction (only w/ FFT):" },
+  { NoOp, 0, "", "", Arg::None,
+    "\nOptions for bulk solvent correction and scaling (only w/ FFT):" },
+  { MaskSpacing, 0, "", "d-mask", Arg::Float,
+    "  --d-mask=NUM  \tSpacing of mask grid (default: same as for model)." },
   { RadiiSet, 0, "", "radii-set", SfCalcArg::Radii,
     "  --radii-set=SET  \tSet of per-element radii, one of: vdw, cctbx, refmac." },
   { Rprobe, 0, "", "r-probe", Arg::Float,
     "  --r-probe=NUM  \tValue added to VdW radius (default: 1.0A)." },
   { Rshrink, 0, "", "r-shrink", Arg::Float,
     "  --r-shrink=NUM  \tValue for shrinking the solvent area (default: 1.1A)." },
+  { MaskFile, 0, "", "solvent-mask", Arg::Required,
+    "  --solvent-mask=FILE  \tUse mask from file instead of calculating it." },
   { Ksolv, 0, "", "ksolv", Arg::Float,
     "  --ksolv=NUM  \tValue (if optimizing: initial value) of k_solv." },
   { Bsolv, 0, "", "bsolv", Arg::Float,
     "  --bsolv=NUM  \tValue (if optimizing: initial value) of B_solv." },
+  { Kov, 0, "", "kov", Arg::Float,
+    "  --kov=NUM  \tValue (if optimizing: initial value) of k_overall." },
   { Baniso, 0, "", "baniso", SfCalcArg::Float6,
     "  --baniso=B11:...:B23 \tAnisotropic scale matrix (6 colon-separated numbers: "
       "B11, B22, B33, B12, B13, B23)." },
@@ -143,6 +156,10 @@ const option::Descriptor Usage[] = {
     " tag (default: F_calc or F_squared_calc)." },
   { PhiLabel, 0, "", "phi", Arg::Required,
     "  --phi=LABEL  \tMTZ column label (default: PHIC)." },
+  { FCanomLabel, 0, "", "fcanom", Arg::Required,
+    "  --fcanom=LABEL  \tMTZ column label for anomalous F (default: FCanom)." },
+  { PHICanomLabel, 0, "", "phicanom", Arg::Required,
+    "  --phicanom=LABEL  \tMTZ column label for anomalous PHI (default: PHICanom)." },
   { 0, 0, 0, 0, 0, 0 }
 };
 
@@ -152,6 +169,8 @@ struct RefFile {
   const char* path = nullptr;
   std::string f_label;
   std::string phi_label;
+  std::string fcanom_label;
+  std::string phicanom_label;
 };
 
 void print_sf(std::complex<double> sf, const gemmi::Miller& hkl) {
@@ -204,21 +223,43 @@ void print_to_stderr(const Comparator& c) {
 
 template<typename Real>
 void write_asudata_to_mtz(const gemmi::AsuData<std::complex<Real>>& asu_data,
-                          const RefFile& file) {
+                          const RefFile& file,
+                          const gemmi::AsuData<std::complex<Real>>* anom_data = nullptr) {
   std::unique_ptr<gemmi::Mtz> output_mtz(new gemmi::Mtz(true));
   output_mtz->set_cell_for_all(asu_data.unit_cell());
   output_mtz->spacegroup = asu_data.spacegroup();
   output_mtz->add_dataset("calculated");
   output_mtz->add_column(file.f_label, 'F', -1, -1, false);
   output_mtz->add_column(file.phi_label, 'P', -1, -1, false);
+
+  bool write_anom = (anom_data != nullptr);
+  if (write_anom) {
+    output_mtz->add_column(file.fcanom_label, 'F', -1, -1, false);
+    output_mtz->add_column(file.phicanom_label, 'P', -1, -1, false);
+  }
+
   output_mtz->title = "Fcalc generated by gemmi";
   output_mtz->nreflections = (int) asu_data.v.size();
-  output_mtz->data.reserve(5 * asu_data.v.size());
+  output_mtz->data.reserve((write_anom ? 7 : 5) * asu_data.v.size());
+
   for (const gemmi::HklValue<std::complex<Real>>& hv : asu_data.v) {
     for (int i = 0; i != 3; ++i)
       output_mtz->data.push_back((float) hv.hkl[i]);
     output_mtz->data.push_back((float) std::abs(hv.value));
     output_mtz->data.push_back((float) gemmi::phase_in_angles(hv.value));
+
+    if (write_anom) {
+      // Find matching hkl in anomalous data
+      auto it = std::lower_bound(anom_data->v.begin(), anom_data->v.end(), hv.hkl);
+      if (it != anom_data->v.end() && it->hkl == hv.hkl) {
+        output_mtz->data.push_back((float) std::abs(it->value));
+        output_mtz->data.push_back((float) gemmi::phase_in_angles(it->value));
+      } else {
+        // Fallback to zero if not found (shouldn't happen with identical grids)
+        output_mtz->data.push_back(0.0f);
+        output_mtz->data.push_back(0.0f);
+      }
+    }
   }
   output_mtz->write_to_file(file.path);
 }
@@ -227,11 +268,13 @@ template<typename Table, typename Real>
 void process_with_fft(const gemmi::Structure& st,
                       gemmi::DensityCalculator<Table, Real>& dencalc,
                       bool mott_bethe,
-                      const gemmi::SolventMasker& masker,
+                      const gemmi::AsuData<std::complex<Real>>* mask_data,
                       gemmi::Scaling<Real>& scaling,
                       bool verbose, const RefFile& file,
                       const gemmi::AsuData<gemmi::ValueSigma<Real>>& scale_to,
-                      const char* map_file) {
+                      const char* map_file,
+                      bool calc_anomalous = false,
+                      double wavelength = 0) {
   // prepare electron density map
   if (verbose) {
     fprintf(stderr, "Preparing electron density on a grid...\n");
@@ -239,7 +282,6 @@ void process_with_fft(const gemmi::Structure& st,
   }
   Timer timer(verbose);
   timer.start();
-  dencalc.set_grid_cell_and_spacegroup(st);
   dencalc.put_model_density_on_grid(st.models[0]);
   timer.print("...took");
   if (map_file) {
@@ -268,7 +310,7 @@ void process_with_fft(const gemmi::Structure& st,
   }
   gemmi::StructureFactorCalculator<Table> calc(st.cell);
   calc.addends = dencalc.addends;
-  gemmi::fileptr_t cache(nullptr, nullptr);
+  gemmi::fileptr_t cache(nullptr, gemmi::needs_fclose{false});
   gemmi::AsuData<std::complex<double>> compared_data;
   if (file.path) {
     if (file.mode == RefFile::Mode::Test) {
@@ -281,41 +323,84 @@ void process_with_fft(const gemmi::Structure& st,
   }
   auto asu_data = sf.prepare_asu_data(dencalc.d_min, dencalc.blur, false, false, mott_bethe);
 
-  gemmi::AsuData<std::complex<Real>> mask_data;
-  if (scaling.use_solvent) {
-    // uses scaling.grid as a temporary array
-    masker.put_mask_on_grid(dencalc.grid, st.models[0]);
-    mask_data = transform_map_to_f_phi(dencalc.grid, /*half_l=*/true)
-                .prepare_asu_data(dencalc.d_min, 0);
-  }
-
   if (scale_to.size() != 0) {
-    scaling.prepare_points(asu_data, scale_to, &mask_data);
+    scaling.prepare_points(asu_data, scale_to, mask_data);
     printf("Calculating scale factors using %zu points...\n", scaling.points.size());
-    scaling.fit_isotropic_b_approximately();
-    //fprintf(stderr, "k_ov=%g B_ov=%g\n", scaling.k_overall, scaling.get_b_overall().u11);
+    if (scaling.b_star.all_zero()) {
+      scaling.fit_isotropic_b_approximately();
+      //fprintf(stderr, "initial k_ov=%g B_ov=%g\n",
+      //        scaling.k_overall, scaling.get_b_overall().u11);
+    } else if (scaling.k_overall == 1.) {
+      scaling.k_overall = scaling.lsq_k_overall();
+      //fprintf(stderr, "initial k_ov=%g\n", scaling.k_overall);
+    }
+    //scaling.fit_b_star_approximately();
     scaling.fit_parameters();
-    gemmi::SMat33<double> b_aniso = scaling.get_b_overall();
     if (scaling.use_solvent)
       fprintf(stderr, "Bulk solvent parameters: k_sol=%g B_sol=%g\n",
               scaling.k_sol, scaling.b_sol);
+    gemmi::SMat33<double> b_aniso = scaling.get_b_overall();
     fprintf(stderr, "k_ov=%g B11=%g B22=%g B33=%g B12=%g B13=%g B23=%g\n",
             scaling.k_overall, b_aniso.u11, b_aniso.u22, b_aniso.u33,
                                b_aniso.u12, b_aniso.u13, b_aniso.u23);
     if (verbose) {
-      std::vector<double> computed = scaling.compute_values();
       Comparator comparator;
-      for (size_t i = 0; i != scaling.points.size(); ++i)
-        comparator.add(computed[i], (double)scaling.points[i].fobs);
+      for (const auto& p : scaling.points)
+        comparator.add(scaling.compute_value(p), (double)p.fobs);
       fprintf(stderr, "After scaling: ");
       print_to_stderr(comparator);
       fprintf(stderr, "\n");
     }
   }
-  scaling.scale_data(asu_data, &mask_data);
+  scaling.scale_data(asu_data, mask_data);
 
   if (file.mode == RefFile::Mode::WriteMtz) {
-    write_asudata_to_mtz(asu_data, file);
+    gemmi::AsuData<std::complex<Real>>* anom_data_ptr = nullptr;
+    gemmi::AsuData<std::complex<Real>> anom_data;
+
+    if (calc_anomalous) {
+      // Calculate anomalous structure factors using f" only
+      gemmi::DensityCalculator<gemmi::ZeroCoef<float>, Real> anom_dencalc;
+      anom_dencalc.d_min = dencalc.d_min;
+      anom_dencalc.rate = dencalc.rate;
+      anom_dencalc.cutoff = dencalc.cutoff;
+      anom_dencalc.blur = dencalc.blur;
+      anom_dencalc.grid.setup_from(st);
+
+      // Set up anomalous form factors (f")
+      auto present_elems = st.models[0].present_elements();
+      if (wavelength > 0) {
+        double energy = gemmi::hc() / wavelength;
+        for (int z = 1; z <= 92; ++z) {
+          if (present_elems[z]) {
+            double fpp;
+            gemmi::cromer_liberman(z, energy, &fpp);
+            anom_dencalc.addends.values[z] = (float)fpp;
+          }
+        }
+      }
+
+      if (verbose) {
+        fprintf(stderr, "Calculating anomalous structure factors (f\")...\n");
+        fflush(stderr);
+      }
+
+      anom_dencalc.put_model_density_on_grid(st.models[0]);
+      gemmi::FPhiGrid<Real> anom_sf = transform_map_to_f_phi(anom_dencalc.grid, /*half_l=*/true);
+      anom_data = anom_sf.prepare_asu_data(anom_dencalc.d_min, anom_dencalc.blur, false, false, false);
+
+      // Validate that anomalous data has compatible unit cell and spacegroup
+      if (!asu_data.unit_cell().approx(anom_data.unit_cell(), 1e-3)) {
+        gemmi::fail("Unit cell mismatch between regular and anomalous data");
+      }
+      if (asu_data.spacegroup() != anom_data.spacegroup()) {
+        gemmi::fail("Spacegroup mismatch between regular and anomalous data");
+      }
+
+      anom_data_ptr = &anom_data;
+    }
+
+    write_asudata_to_mtz(asu_data, file, anom_data_ptr);
   } else if (file.mode == RefFile::Mode::None) {
     for (gemmi::HklValue<std::complex<Real>>& hv : asu_data.v)
       print_sf(hv.value, hv.hkl);
@@ -373,7 +458,7 @@ void print_structure_factors_sm(const gemmi::SmallStructure& small,
   int max_h = int(max_1_d / small.cell.ar);
   int max_k = int(max_1_d / small.cell.br);
   int max_l = int(max_1_d / small.cell.cr);
-  const gemmi::SpaceGroup* sg = small.find_spacegroup();
+  const gemmi::SpaceGroup* sg = small.spacegroup;
   if (!sg)
     sg = &gemmi::get_spacegroup_p1();
   gemmi::ReciprocalAsu asu(sg);
@@ -407,7 +492,7 @@ void print_structure_factors_sm(const gemmi::SmallStructure& small,
   if (file.mode == RefFile::Mode::WriteMtz) {
     asu_data.unit_cell_ = small.cell;
     asu_data.spacegroup_ = sg;
-    write_asudata_to_mtz(asu_data, file);
+    write_asudata_to_mtz<double>(asu_data, file, nullptr);
   }
 }
 
@@ -519,20 +604,6 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
   const gemmi::UnitCell& cell = use_st ? st.cell : small.cell;
   gemmi::StructureFactorCalculator<Table> calc(cell);
 
-  // assign f' given explicitly in a file
-  if (p.options[CifFp]) {
-    if (use_st) {
-      // _atom_type.scat_dispersion_real is almost never used,
-      // so for now we ignore it.
-    } else { // small molecule
-      if (p.options[Verbose])
-        fprintf(stderr, "Using f' read from cif file (%u atom types)\n",
-                (unsigned) small.atom_types.size());
-      for (const gemmi::SmallStructure::AtomType& atom_type : small.atom_types)
-        calc.addends.set(atom_type.element, (float)atom_type.dispersion_real);
-    }
-  }
-
   auto present_elems = use_st ? st.models[0].present_elements()
                               : small.present_elements();
   if (present_elems[(int)gemmi::El::X])
@@ -540,12 +611,37 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
   for (size_t i = 1; i != present_elems.size(); ++i)
     if (present_elems[i] && !Table::has((gemmi::El)i))
       gemmi::fail("Missing form factor for element ", element_name((gemmi::El)i));
-  if (wavelength > 0) {
+
+  if (p.options[CifFp]) { // assign f' given explicitly in a file
+    if (use_st) {
+      // _atom_type.scat_dispersion_real is almost never used,
+      // so for now we ignore it.
+      fprintf(stderr, "WARNING: --ciffp ignored, f' is not read (yet) from mmCIF and PDB.\n");
+    } else { // small molecule
+      if (p.options[Verbose])
+        fprintf(stderr, "Using f' read from cif file (%u atom types)\n",
+                (unsigned) small.atom_types.size());
+      for (const gemmi::SmallStructure::AtomType& atom_type : small.atom_types)
+        calc.addends.set(atom_type.element, (float)atom_type.dispersion_real);
+    }
+  } else if (wavelength > 0) {
+    if (p.options[Verbose])
+      fprintf(stderr, "Wavelength %g A. Using Cromer-Liberman approximation for f'/f\".\n",
+              wavelength);
     double energy = gemmi::hc() / wavelength;
+    bool anomalous = p.options[Anomalous];
     for (int z = 1; z <= 92; ++z)
       if (present_elems[z] && calc.addends.values[z] == 0) {
-        calc.addends.values[z] = (float) gemmi::cromer_liberman(z, energy, nullptr);
+        double f;
+        if (anomalous)
+          gemmi::cromer_liberman(z, energy, &f);
+        else
+          f = gemmi::cromer_liberman(z, energy, nullptr);
+        calc.addends.values[z] = (float)f;
       }
+  } else {
+    if (p.options[Verbose])
+      fprintf(stderr, "Unknown wavelength, f'/f\" not used.\n");
   }
   if (mott_bethe)
     calc.addends.subtract_z();
@@ -587,6 +683,14 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
    file.phi_label = p.options[PhiLabel].arg;
   else
     file.phi_label = "PHIC";
+  if (p.options[FCanomLabel])
+    file.fcanom_label = p.options[FCanomLabel].arg;
+  else
+    file.fcanom_label = "FCanom";
+  if (p.options[PHICanomLabel])
+    file.phicanom_label = p.options[PHICanomLabel].arg;
+  else
+    file.phicanom_label = "PHICanom";
 
   using Real = float;
   gemmi::AsuData<gemmi::ValueSigma<Real>> scale_to;
@@ -613,12 +717,10 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
       scale_to.load_values<2>(gemmi::MtzDataProxy{mtz}, {flabel, flabel});
       for (auto& hkl_value : scale_to.v)
         hkl_value.value.sigma = std::sqrt(hkl_value.value.sigma);
-    } else {
+    } else if (sigma_cutoff >= 0) {
       scale_to.load_values<2>(gemmi::MtzDataProxy{mtz}, {flabel, siglabel});
       size_t size_before = scale_to.size();
-      vector_remove_if(scale_to.v, [=](const gemmi::HklValue<gemmi::ValueSigma<Real>>& x) {
-          return x.value.value <= sigma_cutoff * x.value.sigma;
-      });
+      discard_by_sigma_ratio(scale_to, sigma_cutoff);
       if (p.options[Verbose])
         fprintf(stderr, "Sigma cutoff (F/sigF > %g) excluded %zu out of %zu points.\n",
                 sigma_cutoff, size_before - scale_to.size(), size_before);
@@ -636,6 +738,7 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
       if (p.options[RCut])
         dencalc.cutoff = (float) std::atof(p.options[RCut].arg);
       dencalc.addends = calc.addends;
+      dencalc.grid.setup_from(st);
       if (p.options[Blur]) {
         dencalc.blur = std::atof(p.options[Blur].arg);
       } else if (dencalc.rate < 3) {
@@ -647,9 +750,10 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
         // and on the atomic cutoff radius, and probably it would be too
         // hard to estimate. Here we use the same formula as in Refmac.
         dencalc.set_refmac_compatible_blur(st.models[0]);
-        if (p.options[Verbose])
-          fprintf(stderr, "B_min=%g, B_add=%g\n",
-                  gemmi::get_minimum_b(st.models[0]), dencalc.blur);
+        if (p.options[Verbose]) {
+          double b_min = gemmi::calculate_b_aniso_range(st.models[0]).first;
+          fprintf(stderr, "B_min=%g, B_add=%g\n", b_min, dencalc.blur);
+        }
       }
       gemmi::AtomicRadiiSet radii_choice = gemmi::AtomicRadiiSet::VanDerWaals;
       if (p.options[RadiiSet]) {
@@ -666,6 +770,8 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
         masker.rprobe = std::atof(p.options[Rprobe].arg);
       if (p.options[Rshrink])
         masker.rshrink = std::atof(p.options[Rshrink].arg);
+      if (p.options[MaskSpacing])
+        masker.requested_spacing = std::atof(p.options[MaskSpacing].arg);
 
       gemmi::Scaling<Real> scaling(cell, st.find_spacegroup());
       if (p.options[Ksolv] || p.options[Bsolv] || scale_to.size() != 0) {
@@ -675,6 +781,8 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
         if (p.options[Bsolv])
           scaling.b_sol = std::atof(p.options[Bsolv].arg);
       }
+      if (p.options[Kov])
+        scaling.k_overall = std::atof(p.options[Kov].arg);
       if (p.options[Baniso]) {
         char* endptr = nullptr;
         gemmi::SMat33<double> b_aniso;
@@ -687,8 +795,41 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
         scaling.set_b_overall(b_aniso);
       }
       const char* map_file = p.options[WriteMap] ? p.options[WriteMap].arg : nullptr;
-      process_with_fft(st, dencalc, mott_bethe, masker, scaling,
-                       p.options[Verbose], file, scale_to, map_file);
+
+      // mask's coefficients
+      gemmi::AsuData<std::complex<Real>> mask_data;
+      auto mask_data_ptr = &mask_data;
+      if (scaling.use_solvent) {
+        gemmi::Grid<Real> gr;
+        if (p.options[MaskFile]) {
+          gemmi::Ccp4<Real> ccp4;
+          ccp4.read_ccp4(gemmi::MaybeGzipped(p.options[MaskFile].arg));
+          ccp4.setup(0, gemmi::MapSetup::Full);
+          gr = std::move(ccp4.grid);
+        } else {
+          gr.unit_cell = dencalc.grid.unit_cell;
+          gr.spacegroup = dencalc.grid.spacegroup;
+          double spacing = masker.requested_spacing;
+          if (spacing <= 0)
+            spacing = dencalc.requested_grid_spacing();
+          gr.set_size_from_spacing(spacing, gemmi::GridSizeRounding::Up);
+          masker.put_mask_on_grid(gr, st.models[0]);
+        }
+        if (p.options[Verbose])
+          fprintf(stderr, "Solvent mask: %.1f%% of %d x %d x %d grid.\n",
+                  100. * gr.sum() / gr.point_count(), gr.nu, gr.nv, gr.nw);
+        mask_data = transform_map_to_f_phi(gr, /*half_l=*/true)
+                    .prepare_asu_data(dencalc.d_min, 0);
+      } else {
+        mask_data_ptr = nullptr;
+      }
+
+      bool calc_anom = (p.options[Anomalous] && p.options[ToMtz]);
+      if (calc_anom && wavelength <= 0) {
+        gemmi::fail("Wavelength must be specified (>0) for anomalous calculations with --to-mtz");
+      }
+      process_with_fft(st, dencalc, mott_bethe, mask_data_ptr, scaling,
+                       p.options[Verbose], file, scale_to, map_file, calc_anom, wavelength);
     } else {
       if (p.options[Rate] || p.options[RCut] || p.options[Blur] ||
           p.options[Test])
@@ -804,11 +945,13 @@ void process(const std::string& input, const OptParser& p) {
   char table = p.options[For] ? p.options[For].arg[0] : 'x';
   if (p.options[CifFp] && table != 'x')
     gemmi::fail("Electron scattering has no dispersive part (--ciffp)");
+  if (p.options[Anomalous]) {
+    process_with_table<gemmi::ZeroCoef<float>>(use_st, st, small, wavelength, false, p);
+  }
   if (table == 'x' || table == 'm') {
+    gemmi::IT92<float>::ignore_charge = (table == 'm' || !p.options[UseCharge]);
     if (p.options[NormalizeIt92])
       gemmi::IT92<float>::normalize();
-    if (table == 'm')
-      gemmi::IT92<float>::ignore_charge = true;
     process_with_table<gemmi::IT92<float>>(use_st, st, small, wavelength,
                                            table == 'm', p);
   } else if (table == 'e') {
@@ -824,6 +967,12 @@ int GEMMI_MAIN(int argc, char **argv) {
   OptParser p(EXE_NAME);
   p.simple_parse(argc, argv, Usage);
   p.check_exclusive_group({ToMtz, Test, Compare});
+  if (p.options[MaskFile]) {
+    for (int opt2 : {MaskSpacing, RadiiSet, Rprobe, Rshrink})
+      if (p.options[opt2])
+        std::fprintf(stderr, "WARNING: -%s is ignored when --solvent-mask is given.\n",
+                     p.given_name(opt2));
+  }
   p.require_input_files_as_args();
   try {
     for (int i = 0; i < p.nonOptionsCount(); ++i) {
