@@ -209,6 +209,58 @@ void add_grid_interpolation(nb::class_<Grid<T>, GridBase<T>>& grid) {
               r(i, j, k) = self.interpolate_value(fpos, order);
             }
     }, nb::arg().noconvert(), nb::arg(), nb::arg("order")=1)
+    .def("interpolate_grid_flexible",
+      // Interpolate the moving grid onto the interpolated grid, applying different transforms to 
+      // different regions 
+      [](
+        Gr& interpolated_map,
+        const Gr& moving_map,
+        std::vector<nb::ndarray<int, nb::shape<-1,3>>>& point_arr_vec,
+        std::vector<nb::ndarray<float, nb::shape<-1,3>>>& pos_arr_vec,
+        std::vector<Transform> transform_vec
+      ) {
+        // For each transform in the list, interpolate the positions in the moving map 
+        // corresponding to points in the interpolated map. This method is based on 
+        // contiguous groups of positions in real space, and hence the need to 
+        // separate out the point and pos arrays, to ensure the correct symmetry
+        // copies of of the positions corresponding to a map point are used.
+        // Python code to derive such partitionings of the unit cell around a 
+        // molecule can be found in https://github.com/ConorFWild/pandda_2_gemmi
+        for (std::size_t i=0; i < point_arr_vec.size(); i++){
+            nb::ndarray<int, nb::shape<-1,3>> point_array = point_arr_vec[i];
+            nb::ndarray<float, nb::shape<-1,3>> pos_array = pos_arr_vec[i];
+            const Transform transform = transform_vec[i];
+            auto r_point = point_array.view();
+            auto r_pos = pos_array.view();
+
+            for (std::size_t i=0; i < r_point.shape(0); i++)
+            {
+                // Position
+                Position pos = Position(
+                  r_pos(i,0),
+                  r_pos(i,1),
+                  r_pos(i,2)
+                );
+
+                //transform
+                Position pos_moving = Position(transform.apply(pos));
+
+                // fractionalise
+                Fractional pos_moving_fractional = moving_map.unit_cell.fractionalize(pos_moving);
+
+                // interpolate
+                float interpolated_value = moving_map.interpolate_value(pos_moving_fractional);
+
+                // assign
+                interpolated_map.set_value(
+                    r_point(i, 0),
+                    r_point(i, 1),
+                    r_point(i, 2),
+                    interpolated_value
+                    );
+            };    
+      } 
+      }, nb::arg(), nb::arg().noconvert(), nb::arg().noconvert(), nb::arg())
     ;
 }
 
@@ -271,6 +323,7 @@ void add_grid(nb::module_& m) {
     .def_rw("constant_r", &SolventMasker::constant_r)
     .def_rw("ignore_hydrogen", &SolventMasker::ignore_hydrogen)
     .def_rw("ignore_zero_occupancy_atoms", &SolventMasker::ignore_zero_occupancy_atoms)
+    .def_rw("use_atom_occupancy", &SolventMasker::use_atom_occupancy)
     .def("set_radii", &SolventMasker::set_radii,
          nb::arg("choice"), nb::arg("constant_r")=0.)
     .def("put_mask_on_int8_grid", &SolventMasker::put_mask_on_grid<int8_t>)
@@ -279,7 +332,7 @@ void add_grid(nb::module_& m) {
     ;
   m.def("interpolate_grid", &interpolate_grid<float>,
         nb::arg("dest"), nb::arg("src"), nb::arg("tr"), nb::arg("order")=1);
-  m.def("interpolate_grid_of_aligned_model2", &interpolate_grid_of_aligned_model2<float>,
+  m.def("interpolate_grid_around_model", &interpolate_grid_around_model<float>,
         nb::arg("dest"), nb::arg("src"), nb::arg("tr"),
         nb::arg("dest_model"), nb::arg("radius"), nb::arg("order")=1);
 
