@@ -4,6 +4,7 @@
 
 #include "common.h"
 #include <nanobind/stl/bind_map.h>
+#include <nanobind/stl/map.h>
 #include <nanobind/stl/string.h>
 #include <nanobind/stl/tuple.h>  // for MonLib::match_link
 #include <nanobind/stl/vector.h>
@@ -20,6 +21,7 @@ NB_MAKE_OPAQUE(modifications_type)
 void add_monlib(nb::module_& m) {
   nb::class_<ChemMod> chemmod(m, "ChemMod");
   nb::class_<ChemLink> chemlink(m, "ChemLink");
+  nb::class_<EnerLib> enerlib(m, "EnerLib");
 
   nb::bind_map<monomers_type, rv_ri>(m, "ChemCompMap");
   nb::bind_map<links_type, rv_ri>(m, "ChemLinkMap");
@@ -66,7 +68,18 @@ void add_monlib(nb::module_& m) {
         return "<gemmi.ChemMod " + self.id + ">";
     });
 
-  nb::class_<EnerLib>(m, "EnerLib");  // NOLINT(bugprone-unused-raii)
+  nb::class_<EnerLib::Atom>(enerlib, "Atom")
+    .def_ro("element", &EnerLib::Atom::element)
+    .def_ro("hb_type", &EnerLib::Atom::hb_type)
+    .def_ro("vdw_radius", &EnerLib::Atom::vdw_radius)
+    .def_ro("vdwh_radius", &EnerLib::Atom::vdwh_radius)
+    .def_ro("ion_radius", &EnerLib::Atom::ion_radius)
+    .def_ro("valency", &EnerLib::Atom::valency)
+    .def_ro("sp", &EnerLib::Atom::sp)
+  ;
+  enerlib
+    .def_ro("atoms", &EnerLib::atoms)
+  ;
   nb::class_<MonLib>(m, "MonLib")
     .def(nb::init<>())
     .def_prop_rw("monomer_dir",
@@ -87,8 +100,7 @@ void add_monlib(nb::module_& m) {
          nb::rv_policy::reference_internal)
     .def("test_link", [](const MonLib& self, const ChemLink& link,
                          const std::string& res1, const std::string& atom1,
-                         const std::string& res2, const std::string& atom2)
-    -> std::tuple<bool, const ChemComp::Aliasing*, const ChemComp::Aliasing*> {
+                         const std::string& res2, const std::string& atom2) {
       const ChemComp::Aliasing* aliasing1 = nullptr;
       const ChemComp::Aliasing* aliasing2 = nullptr;
       bool match = (!link.rt.bonds.empty() &&
@@ -98,7 +110,7 @@ void add_monlib(nb::module_& m) {
                     atom_match_with_alias(link.rt.bonds[0].id2.atom, atom2, aliasing2));
       return std::make_tuple(match, aliasing1, aliasing2);
     }, nb::arg("link"), nb::arg("res1"), nb::arg("atom1"), nb::arg("res2"), nb::arg("atom2"),
-         nb::rv_policy::reference_internal)
+       nb::rv_policy::reference_internal)
     .def("add_monomer_if_present", &MonLib::add_monomer_if_present)
     .def("read_monomer_doc", &MonLib::read_monomer_doc)
     .def("read_monomer_cif", &MonLib::read_monomer_cif)
