@@ -57,7 +57,7 @@ void add_dencalc(nb::module_& m, const char* name) {
         if (max_b > 0)
           b = max_b * gemmi::u_to_b();
         float bb = float(b + self.blur);
-        auto coef = Table::get(atom.element, atom.charge);
+        auto coef = Table::get(atom.element, atom.charge, atom.serial);
         auto precal = coef.precalculate_density_iso(bb, self.addends.get(atom.element));
         return self.estimate_radius(precal, bb);
     })
@@ -68,6 +68,7 @@ void add_dencalc(nb::module_& m, const char* name) {
 
 void add_sf(nb::module_& m) {
   nb::class_<gemmi::Addends>(m, "Addends")
+    .def(nb::init<>())
     .def("set", &gemmi::Addends::set)
     .def("get", &gemmi::Addends::get)
     .def("clear", &gemmi::Addends::clear)
@@ -83,11 +84,15 @@ void add_sf(nb::module_& m) {
   using IT92 = gemmi::IT92<float>;
   using C4322 = gemmi::C4322<float>;
   using Neutron92 = gemmi::Neutron92<double>;
+  using CustomCoef = gemmi::CustomCoef<double>;
   add_sfcalc<IT92>(m, "StructureFactorCalculatorX", true);
   add_sfcalc<C4322>(m, "StructureFactorCalculatorE", false);
   add_sfcalc<Neutron92>(m, "StructureFactorCalculatorN", false);
+  //add_sfcalc<CustomCoef>(m, "StructureFactorCalculatorC", false);
   add_dencalc<IT92>(m, "DensityCalculatorX");
   add_dencalc<C4322>(m, "DensityCalculatorE");
   add_dencalc<Neutron92>(m, "DensityCalculatorN");
+  add_dencalc<CustomCoef>(m, "DensityCalculatorC");
+  add_dencalc<gemmi::ZeroCoef<float>>(m, "DensityCalculatorZ");
   m.def("mott_bethe_const", &gemmi::mott_bethe_const);
 }
