@@ -857,11 +857,11 @@ void read_metadata_from_remarks(Structure& st) {
 
 } // anonymous namespace
 
-Structure read_pdb_from_stream(AnyStream& line_reader, const std::string& source,
-                               PdbReadOptions options) {
+void populate_structure_from_pdb_stream(AnyStream& line_reader, const std::string& source,
+                                        Structure& st,
+                                        PdbReadOptions options) {
   if (options.max_line_length <= 0 || options.max_line_length > 120)
     options.max_line_length = 120;
-  Structure st;
   st.input_format = CoorFormat::Pdb;
   st.name = path_basename(source, {".gz", ".pdb"});
   Transform matrix;
@@ -983,7 +983,7 @@ Structure read_pdb_from_stream(AnyStream& line_reader, const std::string& source
       std::string chain_name = read_string(line+10, 2);
       Entity& ent = impl::find_or_add(st.entities, chain_name);
       ent.entity_type = EntityType::Polymer;
-      for (int i = 19; i < 68; i += 4) {
+      for (int i = 19; i < 68 && i < (int)len; i += 4) {
         std::string res_name = read_string(line+i, 3);
         if (!res_name.empty())
           ent.full_sequence.emplace_back(res_name);
@@ -1029,7 +1029,7 @@ Structure read_pdb_from_stream(AnyStream& line_reader, const std::string& source
                is_record_type4(line, "CISPEP")) {
       conn_records.emplace_back(line);
 
-    } else if (is_record_type3(line, "TER")) { // finishes polymer chains
+    } else if (is_record_type3(line, "TER") && !options.ignore_ter) { // finishes polymer chains
       if (!chain || st.ter_status == 'e')
         continue;
       st.ter_status = 'y';
@@ -1236,7 +1236,6 @@ Structure read_pdb_from_stream(AnyStream& line_reader, const std::string& source
     read_metadata_from_remarks(st);
 
   restore_full_ccd_codes(st);
-  return st;
 }
 
 std::vector<Op> read_remark_290(const std::vector<std::string>& raw_remarks) {
