@@ -7,8 +7,8 @@
 
 namespace gemmi {
 
-cif::Document read_cif_gz(const std::string& path) {
-  return cif::read(MaybeGzipped(path));
+cif::Document read_cif_gz(const std::string& path, int check_level) {
+  return cif::read(MaybeGzipped(path), check_level);
 }
 
 bool check_cif_syntax_gz(const std::string& path, std::string* msg) {
@@ -23,8 +23,9 @@ CharArray read_into_buffer_gz(const std::string& path) {
   return read_into_buffer(MaybeGzipped(path));
 }
 
-cif::Document read_cif_from_memory(const char* data, size_t size, const char* name) {
-  return cif::read_memory(data, size, name);
+cif::Document read_cif_from_memory(const char* data, size_t size, const char* name,
+                                   int check_level) {
+  return cif::read_memory(data, size, name, check_level);
 }
 
 cif::Document read_first_block_gz(const std::string& path, size_t limit) {
