@@ -110,7 +110,7 @@ void add_cif_atoms(const Structure& st, cif::Block& block,
 
   std::vector<std::string>& vv = atom_loop.values;
   vv.reserve(atom_site_count * atom_loop.tags.size());
-  std::vector<std::pair<int, const Atom*>> aniso;
+  std::vector<std::tuple<int, int, const Atom*>> aniso;
   int serial = 0;
   for (const Model& model : st.models) {
     for (const Chain& chain : model.chains) {
@@ -156,7 +156,7 @@ void add_cif_atoms(const Structure& st, cif::Block& block,
           if (st.has_d_fraction)
             vv.emplace_back(to_str(atom.fraction));
           if (atom.aniso.nonzero())
-            aniso.emplace_back(serial, &atom);
+            aniso.emplace_back(serial, model.num, &atom);
         }
       }
     }
@@ -167,17 +167,22 @@ void add_cif_atoms(const Structure& st, cif::Block& block,
     cif::Loop& aniso_loop = block.init_mmcif_loop("_atom_site_anisotrop.", {
                                   "id", "type_symbol", "U[1][1]", "U[2][2]",
                                   "U[3][3]", "U[1][2]", "U[1][3]", "U[2][3]"});
+    if (st.models.size() > 1)
+      aniso_loop.tags.push_back("_atom_site_anisotrop.pdbx_PDB_model_num");
     std::vector<std::string>& aniso_val = aniso_loop.values;
     aniso_val.reserve(aniso_loop.tags.size() * aniso.size());
     for (const auto& a : aniso) {
-      aniso_val.emplace_back(std::to_string(a.first));
-      aniso_val.emplace_back(a.second->element.uname());
-      aniso_val.emplace_back(to_str(a.second->aniso.u11));
-      aniso_val.emplace_back(to_str(a.second->aniso.u22));
-      aniso_val.emplace_back(to_str(a.second->aniso.u33));
-      aniso_val.emplace_back(to_str(a.second->aniso.u12));
-      aniso_val.emplace_back(to_str(a.second->aniso.u13));
-      aniso_val.emplace_back(to_str(a.second->aniso.u23));
+      aniso_val.emplace_back(std::to_string(std::get<0>(a)));
+      const Atom* atom = std::get<2>(a);
+      aniso_val.emplace_back(atom->element.uname());
+      aniso_val.emplace_back(to_str(atom->aniso.u11));
+      aniso_val.emplace_back(to_str(atom->aniso.u22));
+      aniso_val.emplace_back(to_str(atom->aniso.u33));
+      aniso_val.emplace_back(to_str(atom->aniso.u12));
+      aniso_val.emplace_back(to_str(atom->aniso.u13));
+      aniso_val.emplace_back(to_str(atom->aniso.u23));
+      if (st.models.size() > 1)
+        aniso_loop.values.push_back(std::to_string(std::get<1>(a)));
     }
   }
 }
@@ -546,8 +551,8 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
                                   {"id", "entity_id", "db_name", "db_code",
                                    "pdbx_db_accession", "pdbx_db_isoform"});
     cif::Loop& seq_loop = block.init_mmcif_loop("_struct_ref_seq.", {
-        "align_id", "ref_id", "pdbx_strand_id",
-        "seq_align_beg", "seq_align_end",
+        "align_id", "ref_id", "pdbx_strand_id", "pdbx_PDB_id_code",
+        "seq_align_beg", "seq_align_end", "pdbx_db_accession",
         "db_align_beg", "db_align_end",
         "pdbx_auth_seq_align_beg", "pdbx_seq_align_beg_ins_code",
         "pdbx_auth_seq_align_end", "pdbx_seq_align_end_ins_code"});
@@ -576,17 +581,28 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
               label_end = span.auth_seq_id_to_label(dbref.seq_end);
             } catch (const std::out_of_range&) {}
           }
+          SeqId begin = dbref.seq_begin;
+          SeqId end = dbref.seq_end;
+          if (!begin.num || !end.num) {
+            if (const Chain* chain = st.models[0].find_chain(strand_id->second))
+              if (ConstResidueGroup polymer = chain->get_polymer()) {
+                begin = polymer.label_seq_id_to_auth(dbref.label_seq_begin);
+                end = polymer.label_seq_id_to_auth(dbref.label_seq_end);
+              }
+          }
           seq_loop.add_row({std::to_string(++counter2),
                             std::to_string(counter),
                             strand_id->second,  // pdbx_strand_id
+                            id,
                             label_begin.str(),
                             label_end.str(),
+                            string_or_qmark(dbref.accession_code),
                             dbref.db_begin.num.str(),
                             dbref.db_end.num.str(),
-                            dbref.seq_begin.num.str(),
-                            pdbx_icode(dbref.seq_begin),
-                            dbref.seq_end.num.str(),
-                            pdbx_icode(dbref.seq_end)});
+                            begin.num.str(),
+                            pdbx_icode(begin),
+                            end.num.str(),
+                            pdbx_icode(end)});
         }
       }
   }
