@@ -7,11 +7,12 @@ BIN=.
 
 echo "\$ gemmi -h" > docs/gemmi-help.txt
 $BIN/gemmi -h >> docs/gemmi-help.txt
-for prog in align blobs cif2json cif2mtz cifdiff contact contents convert crd \
-    ecalc fprime grep h json2cif map map2sf mask merge mondiff mtz mtz2cif \
-    reindex residues rmsz sf2map sfcalc sg tags validate wcn xds2mtz; do
-  echo "\$ gemmi $prog -h" > docs/$prog-help.txt
-  $BIN/gemmi $prog -h >> docs/$prog-help.txt
+for fn in prog/*.cpp; do
+  prog=$(basename $fn .cpp)
+  if grep -q ${prog}_main prog/main.cpp; then
+    echo "\$ gemmi $prog -h" > docs/$prog-help.txt
+    $BIN/gemmi $prog -h >> docs/$prog-help.txt
+  fi
 done
 
 echo "\$ gemmi cif2mtz --print-spec" > docs/cif2mtz-spec.txt
